@@ -17,7 +17,7 @@
 
 #include "qgis_core.h"
 #include <expat.h>
-#include "qgis.h"
+#include "qgis_sip.h"
 #include "qgserror.h"
 #include "qgsfields.h"
 #include <list>
@@ -93,7 +93,8 @@ class CORE_EXPORT QgsGmlSchema : public QObject
       * Currently only recognizes UMN Mapserver GetFeatureInfo GML response.
       * Supports only UTF-8, UTF-16, ISO-8859-1, US-ASCII XML encodings.
       * \param data GML data
-      * \returns true in case of success */
+      * \returns TRUE in case of success
+    */
     bool guessSchema( const QByteArray &data );
 
     //! Gets list of dot separated paths to feature classes parsed from GML or XSD
@@ -116,7 +117,7 @@ class CORE_EXPORT QgsGmlSchema : public QObject
       BoundingBox,
       FeatureMembers, // gml:featureMembers
       FeatureMember, // gml:featureMember
-      Feature,  // feature element containint attrs and geo (inside gml:featureMember)
+      Feature,  // feature element containing attrs and geo (inside gml:featureMember)
       Attribute,
       Geometry
     };
