@@ -28,6 +28,7 @@
 
 class QgsStyle;
 class QgsTemporaryCursorOverride;
+class QgsMessageBar;
 
 #ifndef SIP_RUN
 ///@cond PRIVATE
@@ -36,6 +37,7 @@ class QgsCheckableStyleModel: public QgsStyleProxyModel
     Q_OBJECT
   public:
 
+    explicit QgsCheckableStyleModel( QgsStyleModel *sourceModel, QObject *parent = nullptr, bool readOnly = false );
     explicit QgsCheckableStyleModel( QgsStyle *style, QObject *parent = nullptr, bool readOnly = false );
 
     void setCheckable( bool checkable );
@@ -190,12 +192,12 @@ class GUI_EXPORT QgsStyleManagerDialog : public QDialog, private Ui::QgsStyleMan
     void showHelp();
 
     /**
-     * \deprecated in QGIS 3.6 - has no effect and will be removed in QGIS 4.0
+     * \deprecated since QGIS 3.6 - has no effect and will be removed in QGIS 4.0
      */
     Q_DECL_DEPRECATED void itemChanged( QStandardItem *item ) SIP_DEPRECATED;
 
     /**
-     * Trigerred when the current group (or tag) is changed.
+     * Triggered when the current group (or tag) is changed.
      */
     void groupChanged( const QModelIndex & );
 
@@ -230,7 +232,7 @@ class GUI_EXPORT QgsStyleManagerDialog : public QDialog, private Ui::QgsStyleMan
     void editSmartgroupAction();
 
     /**
-     * \deprecated in QGIS 3.6 - has no effect and will be removed in QGIS 4.0
+     * \deprecated since QGIS 3.6 - has no effect and will be removed in QGIS 4.0
      */
     Q_DECL_DEPRECATED void regrouped( QStandardItem * ) SIP_DEPRECATED;
 
@@ -267,7 +269,7 @@ class GUI_EXPORT QgsStyleManagerDialog : public QDialog, private Ui::QgsStyleMan
     /**
      * Populate combo box with known style items (symbols, color ramps).
      *
-     * \deprecated in QGIS 3.6 - has no effect and will be removed in QGIS 4.0
+     * \deprecated since QGIS 3.6 - has no effect and will be removed in QGIS 4.0
      */
     Q_DECL_DEPRECATED void populateTypes() SIP_DEPRECATED;
 
@@ -275,7 +277,7 @@ class GUI_EXPORT QgsStyleManagerDialog : public QDialog, private Ui::QgsStyleMan
     void populateGroups();
 
     /**
-     * \deprecated in QGIS 3.6 - has no effect and will be removed in QGIS 4.0
+     * \deprecated since QGIS 3.6 - has no effect and will be removed in QGIS 4.0
      */
     Q_DECL_DEPRECATED void setSymbolsChecked( const QStringList & ) SIP_DEPRECATED;
 
@@ -305,12 +307,12 @@ class GUI_EXPORT QgsStyleManagerDialog : public QDialog, private Ui::QgsStyleMan
     bool editColorRamp();
 
     /**
-     * \deprecated in QGIS 3.6 - has no effect and will be removed in QGIS 4.0
+     * \deprecated since QGIS 3.6 - has no effect and will be removed in QGIS 4.0
      */
     Q_DECL_DEPRECATED bool removeSymbol() SIP_DEPRECATED;
 
     /**
-     * \deprecated in QGIS 3.6 - has no effect and will be removed in QGIS 4.0
+     * \deprecated since QGIS 3.6 - has no effect and will be removed in QGIS 4.0
      */
     Q_DECL_DEPRECATED bool removeColorRamp() SIP_DEPRECATED;
 
@@ -330,11 +332,15 @@ class GUI_EXPORT QgsStyleManagerDialog : public QDialog, private Ui::QgsStyleMan
 
     void copyItemsToDefault();
 
+    void copyItem();
+
+    void pasteItem();
+
   private:
     int selectedItemType();
 
     /**
-     * Returns true if the "All" tab is selected.
+     * Returns TRUE if the "All" tab is selected.
      */
     bool allTypesSelected() const;
 
@@ -354,6 +360,7 @@ class GUI_EXPORT QgsStyleManagerDialog : public QDialog, private Ui::QgsStyleMan
                           QWidget *parentWidget, std::unique_ptr<QgsTemporaryCursorOverride> &cursorOverride,
                           bool isImport, const QStringList &importTags, bool addToFavorites, bool ignoreSourceTags );
 
+    QgsMessageBar *mMessageBar = nullptr;
 
     QgsStyle *mStyle = nullptr;
 
@@ -384,8 +391,17 @@ class GUI_EXPORT QgsStyleManagerDialog : public QDialog, private Ui::QgsStyleMan
     //! Menu for the "Add item" toolbutton when in all symbols mode
     QMenu *mMenuBtnAddItemAll = nullptr;
 
+    //! Menu for the "Add item" toolbutton when in label settings mode
+    QMenu *mMenuBtnAddItemLabelSettings = nullptr;
+
+    //! Menu for the "Add item" toolbutton when in legend patch shape mode
+    QMenu *mMenuBtnAddItemLegendPatchShape = nullptr;
+
     QAction *mActionCopyToDefault = nullptr;
 
+    QAction *mActionCopyItem = nullptr;
+    QAction *mActionPasteItem = nullptr;
+
     int mBlockGroupUpdates = 0;
 
     bool mReadOnly = false;
@@ -393,6 +409,15 @@ class GUI_EXPORT QgsStyleManagerDialog : public QDialog, private Ui::QgsStyleMan
     bool mSmartGroupVisible = true;
     QString mBaseName;
 
+    bool addTextFormat();
+    bool editTextFormat();
+
+    bool addLabelSettings( QgsWkbTypes::GeometryType type );
+    bool editLabelSettings();
+
+    bool addLegendPatchShape( QgsSymbol::SymbolType type );
+    bool editLegendPatchShape();
+
     friend class QgsStyleExportImportDialog;
 };
 
