@@ -21,10 +21,6 @@ __author__ = 'Alexander Bruy'
 __date__ = 'September 2013'
 __copyright__ = '(C) 2013, Alexander Bruy'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '2f64a3c4e74022c5555ee19862f034ffacc2b5fe'
-
 import os
 
 from qgis.PyQt.QtGui import QIcon
@@ -41,7 +37,6 @@ pluginPath = os.path.split(os.path.split(os.path.dirname(__file__))[0])[0]
 
 
 class ExtractProjection(GdalAlgorithm):
-
     INPUT = 'INPUT'
     PRJ_FILE_CREATE = 'PRJ_FILE_CREATE'
     WORLD_FILE = 'WORLD_FILE'
@@ -88,9 +83,9 @@ class ExtractProjection(GdalAlgorithm):
         return [self.commandName()]
 
     def processAlgorithm(self, parameters, context, feedback):
-        createPrj = self.parameterAsBool(parameters,
-                                         self.PRJ_FILE_CREATE,
-                                         context)
+        createPrj = self.parameterAsBoolean(parameters,
+                                            self.PRJ_FILE_CREATE,
+                                            context)
         raster = self.parameterAsRasterLayer(parameters, self.INPUT,
                                              context)
         if not raster.dataProvider().name() == 'gdal':
