@@ -21,11 +21,6 @@ __author__ = 'Victor Olaya'
 __date__ = 'November 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '0c5774c0685d4b6f3e92a086234455708d1cd427'
-
-
 from qgis.core import (QgsProcessingException,
                        QgsProcessingParameterVectorLayer,
                        QgsProcessingParameterBoolean,
@@ -35,7 +30,6 @@ from processing.algs.gdal.GdalUtils import GdalUtils
 
 
 class ogrinfo(GdalAlgorithm):
-
     INPUT = 'INPUT'
     SUMMARY_ONLY = 'SUMMARY_ONLY'
     NO_METADATA = 'NO_METADATA'
@@ -76,17 +70,18 @@ class ogrinfo(GdalAlgorithm):
     def getConsoleCommands(self, parameters, context, feedback, executing=True):
         arguments = ['-al']
 
-        if self.parameterAsBool(parameters, self.SUMMARY_ONLY, context):
+        if self.parameterAsBoolean(parameters, self.SUMMARY_ONLY, context):
             arguments.append('-so')
-        if self.parameterAsBool(parameters, self.NO_METADATA, context):
+        if self.parameterAsBoolean(parameters, self.NO_METADATA, context):
             arguments.append('-nomd')
 
         inLayer = self.parameterAsVectorLayer(parameters, self.INPUT, context)
         if inLayer is None:
             raise QgsProcessingException(self.invalidSourceError(parameters, self.INPUT))
 
-        connectionString = GdalUtils.ogrConnectionStringFromLayer(inLayer)
-        arguments.append(connectionString)
+        ogrLayer, layerName = self.getOgrCompatibleSource(self.INPUT, parameters, context, feedback, executing)
+        arguments.append(ogrLayer)
+        arguments.append(layerName)
         return [self.commandName(), GdalUtils.escapeAndJoin(arguments)]
 
     def processAlgorithm(self, parameters, context, feedback):
